@@ -15,7 +15,15 @@
 #ifndef UMUX_H_
 #define UMUX_H_
 
-#ifdef __AVX2__
+#ifdef __AVX512F__
+# include "unitary_calculator_avx512.h"
+  namespace qsim {
+  namespace unitary {
+    template <typename For>
+    using UnitaryCalculator = UnitaryCalculatorAVX512<For>;
+  }
+  }
+#elif __AVX2__
 # include "unitary_calculator_avx.h"
   namespace qsim {
   namespace unitary {
